/* SOURCE CONTROL INFORMATION

	    $RCSfile: bior.c,v $
	    $Revision: 1.3 $
	    Modified $Date: 1994/02/08 23:12:10 $
	    @(#) $Source: /usr2/meade/bior/RCS/bior.c,v $
	    $Author: meade $
*/

/* COMPONENT BIOR	Biorhythm Program
*/

/* DESCRIPTION
This C source code is the conversion of Biorhythmus Version 3.0 by Frank
Kaefer (1987-1994).  In his distribution is a README file with the following
statements::

  "This basic source is free, you can do with it what you want.

   (If you should hack it to run on some other basic, please drop me a line
   by email. Is there someone who wants to port it to C :-) ? And where can
   I get a good basic interpreter for Unix (SunOS 4.X or 5.X) ?)"

Much of the interactive text has been preserved in original form, for
good or ill.

A bug in line 1940 of the basic source file, 'br', has been corrected in
this C version.  The bug was that the graph's centerline was off-center
by one, and should have been placed at position 26, not 25.

A discussion of the biorhythm theory is left for external circles, but,
briefly, this program calculates three different repetitive cycles from
the birthdate.  The three cyclic periods and their associated interpretive
meanings are: every 28 days constitutes a physical cycle, every 28 days
an emotional cycle, and every 33 days a mental cycle.  Each cycle has
a high and low period, designated in the program as "bad condition"
and "good condition" (designation not my own and questionable).  The
various high and low combinations are studied, along with peaks and
valleys, and the beginning and midpoints of the cycles where the
values cross from high to low (known as "critical days") are noted and
studied.
*/

/* PORTABILITY
This source has compiled on the PC using Borland C++ 3.1, and on SVR4
and SVR3 UNIX machines with ANSI C compilers (GNU and ATT versions).
*/

/* ASSUMPTIONS and LIMITATIONS
European-style dates (dd.mm.yy) are used in input and display.

The printer options have been disabled.  Output must be redirected to a
file or to the printer device externally.

The optional default birthdate has been left in and can be set statically
in the header file, bior.h, before compilation.

German comments have not been preserved (sehr gut!).
*/

/* DATA ORGANIZATION
Preserving the original basic-style, many of the main variables are left
as individual file globals.
*/
/****************************************************************/
#if !defined(__MSDOS__)
#ident "@(#) $Id: bior.c,v 1.3 1994/02/08 23:12:10 meade Exp meade $"
#endif

/********************************
 *	Include Files		*
 ********************************/

#include <stdio.h>			/* file I/O definitions		*/
#include <stdlib.h>			/* exit()			*/
#include <string.h>			/* string functions		*/
#include <math.h>			/* transcendental functions	*/
#include <time.h>			/* time/date functions		*/
#include <ctype.h>			/* for tolower() definition	*/

#include "bior.h"			/* local macros and definitions	*/

/****************************************
 *	Global Routine Prototypes	*
 ****************************************/

int	main (int, char **);		/* entry point			*/
void	get_input_data (bool);		/* gather initial input		*/
void	julian_computation (void);	/* perform julian calcs		*/
long	julian_date (DATE *);		/* compute 'Julian' date	*/
void	show_values (void);		/* show biorhythm values	*/
void	show_graph (void);		/* show values by graph		*/
void	print_header (void);		/* print header			*/
void	print_sub_header (void);	/* print basic information	*/

/****************************************
 *	Global Data Structures		*
 ****************************************/

char *weekday[7] =			/* names of days of the week	*/
  {
  "Sunday", "Monday", "Tuesday", "Wednesday",
  "Thursday", "Friday", "Saturday"
  };

					/* number of days in each month	*/
int monthdays[12] = { 31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31 };

DATE	birth;				/* birth data			*/
DATE	today;				/* target data			*/
long	elapsed_days;			/* days old			*/
int	chart_days = DEF_CHART_DAYS;	/* number of days to chart	*/
bool	chart = TRUE;			/* chart output flag		*/
/* bool	printer = FALSE;		/* print flag			*/
/*page*/
/****************************************
 *	Code Section			*
 ****************************************/
/************************************************************************/
/* UNIT MAIN	BIOR Entry Point
*/

/* CALLING SEQUENCE
::
        Call:         bior (from command line)
        Parameters:   command line options not used
        Returns:      standard exit values
::
*/

/* DESCRIPTION
This is the standard program entry point from UNIX.  A great deal of
logic has been left inline in this main routine which would be better
split off into subroutines.  This is, again, to preserve much of the
structure of the original program, written in BASIC.  The code lines
are commented for further explanation of the logic.

The date is enterred in dd.mm.yyyy format.  Note that day, month, and
year can be separated by either periods, slashes, or hyphens (./-).
The original program allowed only periods.
*/

/* ASSUMPTIONS and LIMITATIONS
See DESCRIPTION for a disclaimer on the structure of this routine,
or lack thereof.
*/

/* DATA ORGANIZATION
File global variables are used.
*/
/************************************************************************/

int main (int argc, char *argv[])	/* entry point			*/
  {
  bool	def_input;			/* default input flag		*/

					/* check for default data arg	*/
  def_input = ((argc > 1) && (!strcmp (argv[1], "-d"))) ? TRUE : FALSE;

  get_input_data (def_input);		/* get program data		*/

  julian_computation ();		/* perform julian date calcs	*/

  print_header ();			/* show standard header		*/
  print_sub_header ();			/* additional information	*/

  if (chart)				/* if chart output chosen	*/
    show_graph ();			/* show a graph			*/
  else					/* otherwise			*/
    show_values ();			/* just show daily values	*/

  return 0;				/* done - clean exit		*/
  }
/*page*/
/************************************************************************/
/* UNIT GET_INPUT_DATA	Gather Required Data
*/

/* CALLING SEQUENCE
::
        Call:         void get_input_data (bool def)
        Parameters:   def	default data flag
        Returns:      void
::
*/

/* DESCRIPTION
This routine gathers all data necessary to the program.  It is called
if default settings are not selected with the '-d' command-line
option.
*/

/* ASSUMPTIONS and LIMITATIONS
*/

/* DATA ORGANIZATION
*/
/************************************************************************/

void	get_input_data (bool def)	/* gather initial input		*/
  {
  time_t    dtime;			/* for today's date		*/
  struct tm *todaytm;			/*				*/
  char	    input[80];			/* input buffer			*/

  if (!def)
    {
					/* get target date		*/
    printf ("Date     [<RETURN> for Systemdate] (MM.DD.YY) : ");
    if (gets (input) <= 0)		/* check for error		*/
      {
      perror ("gets");
      exit (1);
      }
    }
  
  else					/* initialize input to NULL	*/
    *input = NULL;

  if (*input)				/* if RETURN not hit		*/
    {
					/* parse out month		*/
    today.month = atol (strtok (input, "./-"));
					/* parse out day		*/
    today.day = atol (strtok (NULL, "./-"));
					/* parse out year		*/
    today.year = atol (strtok (NULL, ""));
    }

  else					/* RETURN hit - get system date	*/
    {
    dtime = time(NULL);			/* get time-stamp		*/
    todaytm = localtime(&dtime);	/* convert to time structure	*/
    today.day = todaytm->tm_mday;	/* get day			*/
    today.month = todaytm->tm_mon + 1;	/* get month			*/
    today.year = todaytm->tm_year;	/* get year			*/
    }

  if (!def)
    {
					/* get birth date		*/
    printf ("Birthday [<RETURN> for %s] (MM.DD.YY) : ", DEF_BIRTHDATE);
    if (gets (input) <= 0)		/* check for error		*/
      {
      perror ("gets");
      exit (2);
      }
    }
  
  if (!*input)				/* use default if RETURN	*/
    strcpy (input, DEF_BIRTHDATE);

					/* parse out target month	*/
  birth.month = atol (strtok (input, "./-"));
					/* parse out target day		*/
  birth.day = atol (strtok (NULL, "./-"));
					/* parse out target year	*/
  birth.year = atol (strtok (NULL, ""));

#ifdef PRINTER_ENABLED
  printf ("Output to screen or printer [<RETURN> for Screen] (s/p) ");
  gets (input);
  printer = (*input == 'p') ? TRUE : FALSE;
#endif

  if (!def)
    {
					/* determine style of output	*/
    printf ("Values for today or Graph   [<RETURN> for Graph]  (v/g) ");
    gets (input);

					/* choose output style		*/
    chart = (tolower(*input) == 'v') ? FALSE : TRUE;
    }

  else					/* default to chart output	*/
    chart = TRUE;

  if ((!def) && chart)
    {
					/* query for number of days	*/
    printf ("Print for how many days? ");
    gets (input);
    chart_days = atoi (input);
    if (chart_days <= 0)
      chart_days = DEF_CHART_DAYS;
    }
  }
/*page*/
/************************************************************************/
/* UNIT JULIAN_COMPUTATION	Compile Julian Data
*/

/* CALLING SEQUENCE
::
        Call:         void julian_computation (void)
        Parameters:   none
        Returns:      void
::
*/

/* DESCRIPTION
This routine is a front end to the routine calculating the
actual target julian dates from the input data.  The number
of days between the dates is calculated from which the sine
waves are constructed.
*/

/* ASSUMPTIONS and LIMITATIONS
*/

/* DATA ORGANIZATION
*/
/************************************************************************/

void julian_computation (void)		/* perform julian calculation	*/
  {
  today.julian = julian_date (&today);	/* compute target julian date	*/

  birth.julian = julian_date (&birth);	/* calculate julian birth date	*/

					/* compute total days lived	*/
  elapsed_days = today.julian - birth.julian;
  }
/*page*/
/************************************************************************/
/* UNIT SHOW_VALUES	Show Daily Biorhythm Values
*/

/* CALLING SEQUENCE
::
        Call:         void show_values (void)
        Parameters:   none
        Returns:      void
::
*/

/* DESCRIPTION
This routine is used to display one day's biorhythm values in
numeric form.  The values are calculated from the number of
elapsed days (previously calculated) between the birth date and
the target date.
*/

/* ASSUMPTIONS and LIMITATIONS
*/

/* DATA ORGANIZATION
*/
/************************************************************************/

void show_values (void)			/* show daily values		*/
  {
  double	ipart;			/* throw-away variable for modf	*/
					/* (see macros in bior.h)	*/

					/* computation by macros	*/
					/* defined in bior.h		*/
  printf ("\nBiorhythm:   %02ld/%02ld/%02ld\n",
				today.day, today.month, today.year);
  printf ("Physical:    %2d%%\n", PHYSICAL);
  printf ("Emotional:   %2d%%\n", EMOTIONAL);
  printf ("Mental:      %2d%%\n", MENTAL);
  }
/*page*/
/************************************************************************/
/* UNIT SHOW_GRAPH	Show Range of Values in Graphic Form
*/

/* CALLING SEQUENCE
::
        Call:         void show_graph (void)
        Parameters:   none
        Returns:      void
::
*/

/* DESCRIPTION
This routine is used to display several day's biorhythm values in
"graphical" form.  Calculations are performed as in the show_values()
routine, but are displayed in a print line as 'P', 'E', and 'M' for
the physical, emotional, and mental cycle values, respectively.
*/

/* ASSUMPTIONS and LIMITATIONS
The chart line has 51 points.  The calculation and placement of the
value markers is limited and fixed upon this number of points (i.e.
it is a simple-minded logic which merely divides the 100% scale by
2, leaving 50 points to plot).
*/

/* DATA ORGANIZATION
*/
/************************************************************************/

void show_graph (void)			/* output a series of days	*/
  {
  int	i;				/* local loop counter		*/
  int	phys, emot, ment;		/* biorhythm values		*/
  long	day, month, year;		/* work variables		*/
  char	graph [GRAPH_WIDTH + 1];	/* graph output line		*/
  double ipart;				/* throw-away variable for modf	*/
					/* (see macros in bior.h)	*/

  graph[GRAPH_WIDTH] = '\0';		/* NULL-terminate graph line	*/

  printf ("P=Physical, E=Emotional, M=Mental\n\n");

					/* and a value-judgment meter	*/
  printf ("%10sbad condition%24sgood condition\n", "", "");

  day = today.day;			/* init local variables		*/
  month = today.month;
  year = today.year;
					/* for all days requested	*/
  for (i = 0; i < chart_days; i++, elapsed_days++)
    {
					/* start by outputing date	*/
    printf ("%02ld/%02ld/%02ld  ", month, day, year);

    phys = PHYSICAL;			/* compute physical value	*/
    emot = EMOTIONAL;			/* compute emotional value	*/
    ment = MENTAL;			/* compute mental value		*/

    phys = (phys + 1) / 2;		/* scale down to graph size	*/
    emot = (emot + 1) / 2;
    ment = (ment + 1) / 2;

    memset (graph, '.', GRAPH_WIDTH);	/* initialize graph line to '.'s*/
    graph [GRAPH_WIDTH / 2] = '|';	/* set mid-stripe		*/
    graph [phys] = 'P';			/* set physical value marker	*/
    graph [emot] = 'E';			/* set emotional value marker	*/
    graph [ment] = 'M';			/* set mental value marker	*/

    printf ("%s\n", graph);		/* print the graph line		*/

					/* increment date by one day	*/
    if (++day > monthdays[month])	/* bump day - if overflow	*/
      {
      day = 1;				/* reset day to 1st of month	*/
      if (++month > 12)			/* inc month - if overflow	*/
	{
	month = 1;			/* reset to January		*/
	year++;				/* of next year			*/
	}
      }
    }
  }
/*page*/
/************************************************************************/
/* UNIT JULIAN_DATE	Calculate the Julian Date
*/

/* CALLING SEQUENCE
::
        Call:         long julian_date (DATE *dp)
        Parameters:   cp	pointer to date structure
        Returns:      julian date (in number of days)
::
*/

/* DESCRIPTION
This routine calculates the number of days from 0 A.D. that have passed
for a particular date.
*/

/* ASSUMPTIONS and LIMITATIONS
Due to various historical manipulations of the Gregorian calendar,
the actual calculations here are not accurate, but they serve the
needs of the program.  Also, "Julian date" means different things
to different people.  The term is used here merely as an available
term and is not meant to cause alarm in purist Julian date circles.
*/

/* DATA ORGANIZATION
*/
/************************************************************************/

long julian_date (DATE *dp)		/* compute the Julian date	*/
  {
  long	yr;				/* work variable		*/
  long	jday = 0L;			/* date variable and return val	*/

  yr = dp->year + 1900;			/* add this century		*/
  if (dp->month <= 2)			/* if in January or February	*/
    jday = (dp->month - 1) * 31;	/* no leap calculation; just	*/
					/* count the days		*/

  else					/* otherwise, harder to figure	*/
    {
    if (!yr % 4)   jday = 1;		/* look for leap year		*/
    if (!yr % 100) jday = 0;		/* century exception		*/
    if (!yr % 400) jday = 1;		/* quad-century exception	*/
					/* compute number of days	*/
    jday += (306 + dp->month - 324) / 10;
    }

  yr--;					/* don't count this year	*/
  jday += yr * 365 + yr / 4;		/* add up years + leap days	*/
  jday -= yr / 100 + yr / 400;		/* and further adjust		*/
  jday += dp->day;			/* add on days			*/

  return jday;				/* return final computation	*/
  }
/*page*/
/************************************************************************/
/* UNIT PRINT_HEADER	Generic Header Information
*/

/* CALLING SEQUENCE
::
        Call:         void print_header (void)
        Parameters:   none
        Returns:      void
::
*/

/* DESCRIPTION
Show a program banner.
*/

/* ASSUMPTIONS and LIMITATIONS
The information here could be combined with the print_sub_header ()
subroutine's information, but is left separate in historic homage
to its predecessor.  It's contents are also inherited from the
original BASIC text.
*/

/* DATA ORGANIZATION
*/
/************************************************************************/

void print_header (void)		/* show static header info	*/
  {
  printf ("\n%30sBIORHYTHM\n%30s=========\n", "", "");
  printf ("%21s(c) 1987-1994 by Frank Kaefer\n", "");
  }
/*page*/
/************************************************************************/
/* UNIT PRINT_SUB_HEADER	Show Program Information
*/

/* CALLING SEQUENCE
::
        Call:         void print_sub_header (void)
        Parameters:   none
        Returns:      void
::
*/

/* DESCRIPTION
This routine shows additional information below the standard program
header.  The birthdate and age in days are displayed.
*/

/* ASSUMPTIONS and LIMITATIONS
*/

/* DATA ORGANIZATION
Printed values are obtained from global variables.
*/
/************************************************************************/

void print_sub_header (void)		/* show further information	*/
  {
  long	day_of_week;			/* day of the week		*/ 

					/* set day-of-week index	*/
  day_of_week = birth.julian - (birth.julian / 7) * 7 + 1;

  printf ("\nThe birthday %ld/%ld/%ld is a %s\n",
			birth.month, birth.day, birth.year,
			weekday[day_of_week]);
  printf ("Age in days: %ld\n", elapsed_days);
  }
